@@ -1,5 +1,5 @@
 ;;; jde-bug.el -- JDEbug Interface
-;; $Revision: 1.6 $ $Date: 2002/10/10 21:47:52 $ 
+;; $Revision: 1.90 $ $Date: 2002/12/19 22:23:44 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
@@ -42,11 +42,6 @@
 (require 'jde-db)
 (require 'efc)
 
-(when jde-xemacsp
-  (defun line-beginning-position ()
-    (point-at-bol))
-  (defun line-end-position ()
-    (point-at-eol)))
 
 (defgroup jde-bug nil
   "JDEbug General Options"
@@ -321,72 +316,34 @@ consuming and slows down stepping through the code."
 
 	"-"
 
-
 	(list
 	 "Display"
 
-	 ["Variable"                  
-	  jde-bug-display-variable 
-	  (and 
-	   (jde-dbs-debugger-running-p)
-	   (jde-dbs-get-target-process))]
-
-
-	 ["Array"                     
-	  jde-bug-display-array
-	  (and 
-	   (jde-dbs-debugger-running-p)
-	   (jde-dbs-get-target-process))]
-
-	 ["Object"                    
-	  jde-bug-display-object
-	  (and 
-	   (jde-dbs-debugger-running-p)
-	   (jde-dbs-get-target-process))
-	  ]
-
-	 ["String"                    
-	  jde-bug-display-string
-	  (and 
-	   (jde-dbs-debugger-running-p)
-	   (jde-dbs-get-target-process))
-	  ]
-
-	 ["Local Variables"           
-	  jde-bug-display-local-variables 
-	  (and 
-	   (jde-dbs-debugger-running-p)
-	   (jde-dbs-get-target-process))] 
-
 	 ["Loaded Classes"            
 	  jde-bug-display-loaded-classes 
 	  (and 
 	   (jde-dbs-debugger-running-p)
 	   (jde-dbs-get-target-process))]
 
-	 ["Threads"                   
-	  jde-bug-show-threads
-	  (and 
-	   (jde-dbs-debugger-running-p)
-	   (jde-dbs-get-target-process))]
-
 	 ["Object Monitors"           
 	  jde-bug-show-object-monitors
 	  (and 
 	   (jde-dbs-debugger-running-p)
 	   (jde-dbs-get-target-process))]
 
-
 	 ["Path Info"                 
 	  jde-bug-display-path-info
 	  (and 
 	   (jde-dbs-debugger-running-p)
 	   (jde-dbs-get-target-process))]
-
-
-
 	 )
 
+        ["Display Variable At Point"
+         jde-bug-display-variable
+         (and
+           (jde-dbs-debugger-running-p)
+           (jde-dbs-get-target-process))]
+
 	["Evaluate Expression"        
 	 jde-bug-evaluate-expression 
 	 (and 
@@ -565,6 +522,10 @@ consuming and slows down stepping through the code."
 	   (jde-dbs-debugger-running-p)
 	   (jde-dbs-get-target-process))]
 	 )
+        ["Show Debug Frame"
+         jde-bug-show-debug-frame
+         (and (jde-dbs-debugger-running-p)
+              (jde-dbs-get-target-process))]
 	"-"
 	["Preferences"                
 	 jde-bug-show-preferences t]
@@ -700,6 +661,7 @@ You can use the notation [f1], [f2], etc., to specify function keys."
     (oset process startupp nil)
     (oset process suspendedp nil)
     (oset process steppablep nil)
+    (setq overlay-arrow-position nil)
     (jde-dbs-cmd-exec run)))
 
 
@@ -1624,6 +1586,58 @@ requests to cancel."
 	 (error "The target process has no outstanding watch requests"))
      (error "There is no active process."))))
 
+(defun jde-bug-show-debug-frame ()
+  "Show or open a new frame with the locals, thread, and CLI buffer."
+  (interactive)
+  (let* ((existing-frame (cdr (assoc "JDebug" (make-frame-names-alist)))))
+    (if existing-frame
+      (progn 
+        (make-frame-visible existing-frame)
+        (raise-frame existing-frame)
+        (select-frame existing-frame))
+      (let* ((process (jde-dbs-get-target-process))
+             (cli-buffer (when (slot-boundp process 'cli-buf)
+                        (oref process cli-buf)))
+             (locals-buffer (when (slot-boundp process 'locals-buf)
+                           (oref process locals-buf)))
+             (threads-buffer (when (slot-boundp process 'threads-buf)
+                            (oref process threads-buf)))
+             (msg-buffer (when (slot-boundp process 'msg-buf)
+                        (oref process msg-buf)))
+             (frame (new-frame '((name . "JDebug") (minibuffer . nil))))
+             (height (/ (frame-height frame) (count-if 'identity
+                                                       (list cli-buffer
+                                                             locals-buffer
+                                                             threads-buffer
+                                                             msg-buffer))))
+             (init-frame (selected-frame))
+             (init-config (current-window-configuration))
+             (prev-window))
+        (save-excursion
+          (select-frame frame)
+          ;; msg buffer should always be there, if not this could break
+          (switch-to-buffer msg-buffer)
+          (setq prev-window (get-buffer-window msg-buffer))
+          (when locals-buffer
+            (let ((locals-win (split-window prev-window height)))
+              (setq prev-window locals-win)
+              (when (not jde-bug-local-variables)
+                (jde-bug-toggle-local-variables)
+                (if (featurep 'xemacs)
+                  (sleep-for 0.1)
+                  (sleep-for 0 100)))
+              (set-window-buffer locals-win locals-buffer)))
+          (when threads-buffer
+            (let ((threads-win (split-window prev-window height)))
+              (setq prev-window threads-win)
+              (jde-bug-show-threads)
+              (set-window-buffer threads-win threads-buffer)))
+          (when cli-buffer
+            (let ((cli-win (split-window prev-window height)))
+              (set-window-buffer cli-win cli-buffer)))
+          (select-frame init-frame)
+          (set-window-configuration init-config))))))
+
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;;
@@ -1635,13 +1649,10 @@ requests to cancel."
   (interactive)
   (if (not (jde-dbs-target-process-runnable-p))
       (error "No target process or process is not suspended."))
-  (let ((qualified-expr 
-	 (oref (semantic-analyze-current-context (point)) prefix)))
-    (if qualified-expr
-	(jde-bug-evaluate-expression 
-	 (mapconcat 'identity qualified-expr ".")))))
-	 
-
+  ;; Previously we used semantic. I recently have been having problems
+  ;; with it, so I just replaced it with word-at-point
+  (let ((qualified-expr (thing-at-point 'word)))
+    (jde-bug-evaluate-expression qualified-expr)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;;
@@ -1673,160 +1684,13 @@ any variables in scope in the program being debugged."
 	 (result
 	  (jde-dbs-cmd-exec evaluate-command)))
     (if result
-	(let* ((object-p (equal (length result) 3))
-	       (type  (nth 0 result))
+	(let* ((type  (nth 0 result))
 	       (value (nth 1 result))
-	       (gc    (if object-p 
-			  (if (nth 2 result)
-			      "(garbage collected)"
-			    "")))
-	       (formatted-result 
-		(if object-p
-		    (format "<%s:%s> %s" type value gc)
-		  (format "%s (%s)" value type))))
-	  (jde-dbs-proc-display-debug-message
-	   process
-	   (format "\"%s\" = %s" expression formatted-result))
-	  (message formatted-result))
+               (buf (get-buffer-create (concat "Expression: " expression))))
+            (jde-dbo-view-var-in-buf (jde-dbs-objectify-value result)
+                                     expression process t buf t)
+            (pop-to-buffer buf (split-window nil (- (window-height) 4))))
       (message "Error: could not evaluate \"%s\"." expression))))
-
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;                                                                            ;;
-;; Display Array Command                                                      ;;
-;;                                                                            ;;
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(defun jde-bug-get-array-length (array)
-  (let* ((process (jde-dbs-get-target-process))
-	 (get-array-command
-	  (jde-dbs-get-array 
-	   (format "get_array_length %d" (oref array id))
-	   :process process
-	   :array array)))
-	 (jde-dbs-cmd-exec get-array-command)
-	 (if (slot-boundp array 'length)
-	     (oref array length))))
-
-(defun jde-bug-get-array-elements (array length)
-  (let* ((process (jde-dbs-get-target-process))
-	 (get-array-command
-	  (jde-dbs-get-array 
-	   (format "get_array_elements %d" (oref array id))
-	   :process process
-	   :array array
-	   :index 0
-	   :length length)))
-	 (jde-dbs-cmd-exec get-array-command)))
-
-
-(defun jde-bug-display-array (array-id)
-  (interactive
-   "nArray ID: ")
-
-  (if (not (jde-dbs-target-process-runnable-p))
-      (error "No target process or process is not suspended."))
-
-  (let* ((array
-	  (jde-dbs-java-array 
-	   (format "array %d" array-id) :id array-id))
-	 (array-length
-	  (jde-bug-get-array-length array)))
-    (if array-length
-	(if (> array-length 0)
-	    (progn
-	      (jde-bug-get-array-elements array array-length)
-	      (jde-dbs-proc-display-debug-message
-	       (jde-dbs-get-target-process) 
-	       (jde-dbs-java-obj-to-string array)))
-	  (jde-dbs-proc-display-debug-message
-	   (jde-dbs-get-target-process)
-	   (format "Array %d has no elements." array-id)))      
-      (error "Could not get array elements."))))
-
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;                                                                            ;;
-;; Display Object Command                                                     ;;
-;;                                                                            ;;
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(defun jde-bug-display-object (object-id)
-  (interactive
-   "nObject ID: ")
-  (let* ((process (jde-dbs-get-target-process))
-	 (get-object-command
-	  (jde-dbs-get-object
-	   (format "get_object %d" object-id)
-	   :process process
-	   :object-id object-id))
-	 (object
-	  (jde-dbs-cmd-exec get-object-command)))
-    (if object
-	(jde-dbs-proc-display-debug-message
-	 (jde-dbs-get-target-process)
-	 (jde-dbs-java-obj-to-string object))
-      (error "Could not get object"))))
- 
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;                                                                            ;;
-;; Display String Command                                                     ;;
-;;                                                                            ;;
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(defun jde-bug-display-string (object-id) 
-  (interactive
-   "nObject ID: ")
-
-  (if (not (jde-dbs-target-process-runnable-p))
-      (error "No target process or process is not suspended."))
-
-  (let* ((process (jde-dbs-get-target-process))
-	 (get-string-command
-	  (jde-dbs-get-string
-	   (format "get_string %d" object-id)
-	   :process process
-	   :object-id object-id))
-	 (string
-	  (jde-dbs-cmd-exec get-string-command)))
-    (if string
-	(jde-dbs-proc-display-debug-message
-	 (jde-dbs-get-target-process)
-	 (format "string %d = %s" object-id string))
-      (error "Could not get object"))))
- 
-
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;                                                                            ;;
-;; Display Local Variables Command                                            ;;
-;;                                                                            ;;
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-
-(defun jde-bug-display-local-variables () 
-  (interactive)
-
-  (if (not (jde-dbs-target-process-runnable-p))
-      (error "No target process or process is not suspended."))
-
-  (let* ((process (jde-dbs-get-target-process))
-	 (state-info (oref process state-info))
-	 (thread-id (oref state-info thread-id))
-	 (get-locals-command
-	  (jde-dbs-get-locals
-	   (format "get_locals t%d" thread-id)
-	   :process process
-	   :thread-id thread-id))
-	 (result
-	  (jde-dbs-cmd-exec get-locals-command)))
-    (if result
-	(mapc
-	 (lambda (var) 
-	   (jde-dbs-proc-display-debug-message
-	    (jde-dbs-get-target-process)
-	    (jde-dbs-java-variable-to-string var)))
-	 result)	    
-      (error "Could not get locals"))))
  
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;                                                                            ;;
@@ -2593,11 +2457,26 @@ of the variable `jde-bug-stack-info'"
 (provide 'jde-bug)
 
 ;; $Log: jde-bug.el,v $
-;; Revision 1.6  2002/10/10 21:47:52  andyp
-;; misc fixes
+;; Revision 1.90  2002/12/19 22:23:44  ahyatt
+;; Should have used slot-boundp instead of slot-existsp
 ;;
-;; Revision 1.5  2002/09/14 03:20:53  andyp
-;; sync to jde 2.2.9beta12
+;; Revision 1.88  2002/12/08 20:40:35  ahyatt
+;; Removed most items from the debug "Display" menu, and created two new
+;; menu items, a "Display variable at point", and "Show debug frame".
+;; The "Display variable at point", and the "Evaluate expression" command
+;; shows a tree-view of the returned variable.  The "Show debug frame"
+;; will put up a debug frame of all debug buffers.
+;;
+;; Revision 1.87  2002/12/07 17:09:44  jslopez
+;; Fixes bug ID 4. The overlay arrow nows get cleared after using
+;; jde-bug-continue.
+;;
+;; Revision 1.86  2002/10/30 13:01:41  paulk
+;; Moved XEmacs compatibility fix for beginning/end-of-line functions
+;; from this file to jde-db.el
+;;
+;; Revision 1.85  2002/10/16 04:55:43  paulk
+;; Adds a menu entry to JDEbug to toggle the stack display. Thanks to Andy Piper.
 ;;
 ;; Revision 1.84  2002/09/11 03:37:43  paulk
 ;; Fixes typo in code.
